@@ -2,6 +2,7 @@
 # -*- coding: utf-8 -*-
 # This file is part of the COVID-surge application.
 # https://github/dpploy/covid-surge
+# Valmor F. de Almeida dealmeidavf@gmail.com
 '''
 US COVID-19 surge period analysis.
 
@@ -29,7 +30,7 @@ def main():
     print('*                        Single State                              *')
     print('********************************************************************')
 
-    name = 'Massachusetts'
+    name = 'North Carolina'
     print(name)
     print('')
 
